@@ -34,9 +34,14 @@ const (
 	CleanupTmp  = "tmp"
 )
 
-var (
-	DryRun = flag.Bool("dry-run", false, "do no delete files during cleanup, just show them")
-)
+var DryRun *bool
+
+func init() {
+	if CmdName() != CmdNameRedoCleanup {
+		return
+	}
+	DryRun = flag.Bool("n", false, "do no delete files during cleanup, just show them")
+}
 
 func redoDirClean(root, what string) error {
 	root, err := filepath.Abs(root)
