@@ -126,7 +126,7 @@ func isOOD(cwd, tgtOrig string, level int, seen map[string]struct{}) (bool, erro
 		}
 		theirInode, err := inodeFromRec(m)
 		if err != nil {
-			return ood, TgtErr{tgtOrig, fmt.Errorf("invalid format of .rec: %v", err)}
+			return ood, TgtErr{tgtOrig, fmt.Errorf("invalid format of .rec: %w", err)}
 		}
 		theirHsh := m["Hash"]
 		trace(CDebug, "ood: %s%s -> %s: checking", indent, tgtOrig, dep)
